--[[
Zotwee_Garrison addon, 1.0.0
http://zotwee.info/
]]

local addonName = "Zotwee_Garrison"

_G[addonName] = LibStub("AceAddon-3.0"):NewAddon(addonName, "AceConsole-3.0", "AceEvent-3.0", "AceComm-3.0", "AceTimer-3.0")
local Zaddon = _G[addonName]

local parse = LibStub("LibParse")

Zaddon.version = "1.0.0"

local private = {}

local timeoffset = 0

local AddonDB_Defaults = {
  global = {
    char = {
      ['*'] = {       -- ["Realm.Name"] 
        lastUpdate = nil,
        lastResourceCollection = nil,
        Buildings = {},       -- List of buildings
        Followers = {},       -- List of followers
        ActiveMissions = {},      -- List of active/in-progress missions
        ActiveMissionsInfo = {},  -- List of start times for active/in-progress missions
      }
    }
  }
}


function Zaddon:OnDisable()
  Zaddon:UnregisterEvent("PLAYER_ALIVE")
  Zaddon:UnregisterEvent("GARRISON_FOLLOWER_ADDED")
  Zaddon:UnregisterEvent("GARRISON_FOLLOWER_LIST_UPDATE")
  Zaddon:UnregisterEvent("GARRISON_FOLLOWER_REMOVED")
  Zaddon:UnregisterEvent("GARRISON_BUILDING_ACTIVATED")
  Zaddon:UnregisterEvent("GARRISON_BUILDING_UPDATE")
  Zaddon:UnregisterEvent("GARRISON_BUILDING_REMOVED")
  Zaddon:UnregisterEvent("SHOW_LOOT_TOAST")
  Zaddon:UnregisterEvent("GARRISON_MISSION_LIST_UPDATE")
  Zaddon:UnregisterEvent("GARRISON_MISSION_STARTED")
  Zaddon:UnregisterEvent("GARRISON_MISSION_FINISHED")
  Zaddon:UnregisterEvent("ADDON_LOADED")
end

local inspect ={
  _VERSION = 'inspect.lua 3.0.0',
  _URL     = 'http://github.com/kikito/inspect.lua',
  _DESCRIPTION = 'human-readable representations of tables',
  _LICENSE = [[
    MIT LICENSE

    Copyright (c) 2013 Enrique García Cota

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the
    "Software"), to deal in the Software without restriction, including
    without limitation the rights to use, copy, modify, merge, publish,
    distribute, sublicense, and/or sell copies of the Software, and to
    permit persons to whom the Software is furnished to do so, subject to
    the following conditions:

    The above copyright notice and this permission notice shall be included
    in all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
    OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
    MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
    IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
    CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
    TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
    SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  ]]
}

inspect.KEY       = setmetatable({}, {__tostring = function() return 'inspect.KEY' end})
inspect.METATABLE = setmetatable({}, {__tostring = function() return 'inspect.METATABLE' end})

-- Apostrophizes the string if it has quotes, but not aphostrophes
-- Otherwise, it returns a regular quoted string
local function smartQuote(str)
  if str:match('"') and not str:match("'") then
    return "'" .. str .. "'"
  end
  return '"' .. str:gsub('"', '\\"') .. '"'
end

local controlCharsTranslation = {
  ["\a"] = "\\a",  ["\b"] = "\\b", ["\f"] = "\\f",  ["\n"] = "\\n",
  ["\r"] = "\\r",  ["\t"] = "\\t", ["\v"] = "\\v"
}

local function escape(str)
  local result = str:gsub("\\", "\\\\"):gsub("(%c)", controlCharsTranslation)
  return result
end

local function isIdentifier(str)
  return type(str) == 'string' and str:match( "^[_%a][_%a%d]*$" )
end

local function isSequenceKey(k, length)
  return type(k) == 'number'
     and 1 <= k
     and k <= length
     and math.floor(k) == k
end

local defaultTypeOrders = {
  ['number']   = 1, ['boolean']  = 2, ['string'] = 3, ['table'] = 4,
  ['function'] = 5, ['userdata'] = 6, ['thread'] = 7
}

local function sortKeys(a, b)
  local ta, tb = type(a), type(b)

  -- strings and numbers are sorted numerically/alphabetically
  if ta == tb and (ta == 'string' or ta == 'number') then return a < b end

  local dta, dtb = defaultTypeOrders[ta], defaultTypeOrders[tb]
  -- Two default types are compared according to the defaultTypeOrders table
  if dta and dtb then return defaultTypeOrders[ta] < defaultTypeOrders[tb]
  elseif dta     then return true  -- default types before custom ones
  elseif dtb     then return false -- custom types after default ones
  end

  -- custom types are sorted out alphabetically
  return ta < tb
end

local function getNonSequentialKeys(t)
  local keys, length = {}, #t
  for k,_ in pairs(t) do
    if not isSequenceKey(k, length) then table.insert(keys, k) end
  end
  table.sort(keys, sortKeys)
  return keys
end

local function getToStringResultSafely(t, mt)
  local __tostring = type(mt) == 'table' and rawget(mt, '__tostring')
  local str, ok
  if type(__tostring) == 'function' then
    ok, str = pcall(__tostring, t)
    str = ok and str or 'error: ' .. tostring(str)
  end
  if type(str) == 'string' and #str > 0 then return str end
end

local maxIdsMetaTable = {
  __index = function(self, typeName)
    rawset(self, typeName, 0)
    return 0
  end
}

local idsMetaTable = {
  __index = function (self, typeName)
    local col = setmetatable({}, {__mode = "kv"})
    rawset(self, typeName, col)
    return col
  end
}

local function countTableAppearances(t, tableAppearances)
  tableAppearances = tableAppearances or setmetatable({}, {__mode = "k"})

  if type(t) == 'table' then
    if not tableAppearances[t] then
      tableAppearances[t] = 1
      for k,v in pairs(t) do
        countTableAppearances(k, tableAppearances)
        countTableAppearances(v, tableAppearances)
      end
      countTableAppearances(getmetatable(t), tableAppearances)
    else
      tableAppearances[t] = tableAppearances[t] + 1
    end
  end

  return tableAppearances
end

local copySequence = function(s)
  local copy, len = {}, #s
  for i=1, len do copy[i] = s[i] end
  return copy, len
end

local function makePath(path, ...)
  local keys = {...}
  local newPath, len = copySequence(path)
  for i=1, #keys do
    newPath[len + i] = keys[i]
  end
  return newPath
end

local function processRecursive(process, item, path)
  if item == nil then return nil end

  local processed = process(item, path)
  if type(processed) == 'table' then
    local processedCopy = {}
    local processedKey

    for k,v in pairs(processed) do
      processedKey = processRecursive(process, k, makePath(path, k, inspect.KEY))
      if processedKey ~= nil then
        processedCopy[processedKey] = processRecursive(process, v, makePath(path, processedKey))
      end
    end

    local mt  = processRecursive(process, getmetatable(processed), makePath(path, inspect.METATABLE))
    setmetatable(processedCopy, mt)
    processed = processedCopy
  end
  return processed
end


-------------------------------------------------------------------

local Inspector = {}
local Inspector_mt = {__index = Inspector}

function Inspector:puts(...)
  local args   = {...}
  local buffer = self.buffer
  local len    = #buffer
  for i=1, #args do
    len = len + 1
    buffer[len] = tostring(args[i])
  end
end

function Inspector:down(f)
  self.level = self.level + 1
  f()
  self.level = self.level - 1
end

function Inspector:tabify()
  self:puts(self.newline, string.rep(self.indent, self.level))
end

function Inspector:alreadyVisited(v)
  return self.ids[type(v)][v] ~= nil
end

function Inspector:getId(v)
  local tv = type(v)
  local id = self.ids[tv][v]
  if not id then
    id              = self.maxIds[tv] + 1
    self.maxIds[tv] = id
    self.ids[tv][v] = id
  end
  return id
end

function Inspector:putKey(k)
  if isIdentifier(k) then return self:puts(k) end
  self:puts("[")
  self:putValue(k)
  self:puts("]")
end

function Inspector:putTable(t)
  if t == inspect.KEY or t == inspect.METATABLE then
    self:puts(tostring(t))
  elseif self:alreadyVisited(t) then
    self:puts('<table ', self:getId(t), '>')
  elseif self.level >= self.depth then
    self:puts('{...}')
  else
    if self.tableAppearances[t] > 1 then self:puts('<', self:getId(t), '>') end

    local nonSequentialKeys = getNonSequentialKeys(t)
    local length            = #t
    local mt                = getmetatable(t)
    local toStringResult    = getToStringResultSafely(t, mt)

    self:puts('{')
    self:down(function()
      if toStringResult then
        self:puts(' -- ', escape(toStringResult))
        if length >= 1 then self:tabify() end
      end

      local count = 0
      for i=1, length do
        if count > 0 then self:puts(',') end
        self:puts(' ')
        self:putValue(t[i])
        count = count + 1
      end

      for _,k in ipairs(nonSequentialKeys) do
        if count > 0 then self:puts(',') end
        self:tabify()
        self:putKey(k)
        self:puts(' = ')
        self:putValue(t[k])
        count = count + 1
      end

      if mt then
        if count > 0 then self:puts(',') end
        self:tabify()
        self:puts('<metatable> = ')
        self:putValue(mt)
      end
    end)

    if #nonSequentialKeys > 0 or mt then -- result is multi-lined. Justify closing }
      self:tabify()
    elseif length > 0 then -- array tables have one extra space before closing }
      self:puts(' ')
    end

    self:puts('}')
  end
end

function Inspector:putValue(v)
  local tv = type(v)

  if tv == 'string' then
    self:puts(smartQuote(escape(v)))
  elseif tv == 'number' or tv == 'boolean' or tv == 'nil' then
    self:puts(tostring(v))
  elseif tv == 'table' then
    self:putTable(v)
  else
    self:puts('<',tv,' ',self:getId(v),'>')
  end
end

-------------------------------------------------------------------

function Zaddon:inspect(root, options)
  options       = options or {}

  local depth   = options.depth   or math.huge
  local newline = options.newline or '\n'
  local indent  = options.indent  or '  '
  local process = options.process

  if process then
    root = processRecursive(process, root, {})
  end

  local inspector = setmetatable({
    depth            = depth,
    buffer           = {},
    level            = 0,
    ids              = setmetatable({}, idsMetaTable),
    maxIds           = setmetatable({}, maxIdsMetaTable),
    newline          = newline,
    indent           = indent,
    tableAppearances = countTableAppearances(root)
  }, Inspector_mt)

  inspector:putValue(root)

  return table.concat(inspector.buffer)
end

setmetatable(inspect, { __call = function(_, ...) return Zaddon:inspect(...) end })

local BUILDING_ALCHEMY = "AlchemyLab"
local BUILDING_BARN = "Barn"
local BUILDING_BARRACKS = "Barracks"
local BUILDING_DWARVEN_BUNKER = "DwarvenBunker"
local BUILDING_ENCHANTERS_STUDY = "EnchantersStudy"
local BUILDING_ENGINEERING_WORKS = "EngineeringWorks"
local BUILDING_FISHING_SHACK = "FishingShack"
local BUILDING_GEM_BOUTIQUE = "GemBoutique"
local BUILDING_GLADIATORS_SANCTUM = "GladiatorsSanctum"
local BUILDING_GNOMISH_GEARWORKS = "GnomishGearworks"
local BUILDING_HERB_GARDEN = "HerbGarden"
local BUILDING_LUMBER_MILL = "LumberMill"
local BUILDING_LUNARFALL_EXCAVATION = "LunarfallExcavation"
local BUILDING_LUNARFALL_INN = "LunarfallInn"
local BUILDING_MAGE_TOWER = "MageTower"
local BUILDING_MENAGERIE = "Menagerie"
local BUILDING_SALVAGE_YARD = "SalvageYard"
local BUILDING_SCRIBES_QUARTERS = "ScribesQuarters"
local BUILDING_STABLES = "Stables"
local BUILDING_STOREHOUSE = "Storehouse"
local BUILDING_TAILORING_EMPORIUM = "TailoringEmporium"
local BUILDING_THE_FORGE = "TheForge"
local BUILDING_THE_TANNERY = "TheTannery"
local BUILDING_TRADING_POST = "TradingPost"

local BUILDING_TOWN_HALL = 0

local buildingTypes = {
  [76] = BUILDING_ALCHEMY,
  [119] = BUILDING_ALCHEMY,
  [120] = BUILDING_ALCHEMY,
  [24] = BUILDING_BARN,
  [25] = BUILDING_BARN,
  [133] = BUILDING_BARN,
  [26] = BUILDING_BARRACKS,
  [27] = BUILDING_BARRACKS,
  [28] = BUILDING_BARRACKS,
  [8] = BUILDING_DWARVEN_BUNKER,
  [9] = BUILDING_DWARVEN_BUNKER,
  [10] = BUILDING_DWARVEN_BUNKER,
  [93] = BUILDING_ENCHANTERS_STUDY,
  [125] = BUILDING_ENCHANTERS_STUDY,
  [126] = BUILDING_ENCHANTERS_STUDY,
  [91] = BUILDING_ENGINEERING_WORKS,
  [123] = BUILDING_ENGINEERING_WORKS,
  [124] = BUILDING_ENGINEERING_WORKS,
  [64] = BUILDING_FISHING_SHACK,
  [134] = BUILDING_FISHING_SHACK,
  [135] = BUILDING_FISHING_SHACK,
  [96] = BUILDING_GEM_BOUTIQUE,
  [131] = BUILDING_GEM_BOUTIQUE,
  [132] = BUILDING_GEM_BOUTIQUE,
  [159] = BUILDING_GLADIATORS_SANCTUM,
  [160] = BUILDING_GLADIATORS_SANCTUM,
  [161] = BUILDING_GLADIATORS_SANCTUM,
  [162] = BUILDING_GNOMISH_GEARWORKS,
  [163] = BUILDING_GNOMISH_GEARWORKS,
  [164] = BUILDING_GNOMISH_GEARWORKS,
  [29] = BUILDING_HERB_GARDEN,
  [136] = BUILDING_HERB_GARDEN,
  [137] = BUILDING_HERB_GARDEN,
  [40] = BUILDING_LUMBER_MILL,
  [41] = BUILDING_LUMBER_MILL,
  [138] = BUILDING_LUMBER_MILL,
  [61] = BUILDING_LUNARFALL_EXCAVATION,
  [62] = BUILDING_LUNARFALL_EXCAVATION,
  [63] = BUILDING_LUNARFALL_EXCAVATION,
  [34] = BUILDING_LUNARFALL_INN,
  [35] = BUILDING_LUNARFALL_INN,
  [36] = BUILDING_LUNARFALL_INN,
  [37] = BUILDING_MAGE_TOWER,
  [38] = BUILDING_MAGE_TOWER,
  [39] = BUILDING_MAGE_TOWER,
  [42] = BUILDING_MENAGERIE,
  [167] = BUILDING_MENAGERIE,
  [168] = BUILDING_MENAGERIE,
  [52] = BUILDING_SALVAGE_YARD,
  [140] = BUILDING_SALVAGE_YARD,
  [141] = BUILDING_SALVAGE_YARD,
  [95] = BUILDING_SCRIBES_QUARTERS,
  [129] = BUILDING_SCRIBES_QUARTERS,
  [130] = BUILDING_SCRIBES_QUARTERS,
  [65] = BUILDING_STABLES,
  [66] = BUILDING_STABLES,
  [67] = BUILDING_STABLES,
  [51] = BUILDING_STOREHOUSE,
  [142] = BUILDING_STOREHOUSE,
  [143] = BUILDING_STOREHOUSE,
  [94] = BUILDING_TAILORING_EMPORIUM,
  [127] = BUILDING_TAILORING_EMPORIUM,
  [128] = BUILDING_TAILORING_EMPORIUM,
  [60] = BUILDING_THE_FORGE,
  [117] = BUILDING_THE_FORGE,
  [118] = BUILDING_THE_FORGE,
  [90] = BUILDING_THE_TANNERY,
  [121] = BUILDING_THE_TANNERY,
  [122] = BUILDING_THE_TANNERY,
  [111] = BUILDING_TRADING_POST,
  [144] = BUILDING_TRADING_POST,
  [145] = BUILDING_TRADING_POST,
}

local function GetNumUncollectedResources(from)
  -- no known collection time (alt never logged in) .. return 0
  if not from then return 0 end
  
  local age = time() - from
  local resources = math.floor(age / 600)   -- 10 minutes = 1 resource
  
  -- cap at 500
  if resources > 500 then
    resources = 500
  end
  return resources
end

-- *** Scanning functions ***
--http://wowprogramming.com/docs/api/C_Garrison.GetOwnedBuildingInfo
--http://wowprogramming.com/docs/api/C_Garrison.GetLandingPageShipmentInfo
local function ScanBuildings()
  
  if not C_Garrison.IsOnGarrisonMap() then return end

  local plots = C_Garrison.GetPlots()

  -- to avoid deleting previously saved data when the game is not ready to deliver information
  -- exit if no data is available
  if not plots or #plots == 0 then return end

  local buildings = {}
  
  -- Scan Town Hall
  local level = C_Garrison.GetGarrisonInfo()
  
  buildings[BUILDING_TOWN_HALL] = { rank = level }
  
  -- Scan other buildings
  for i = 1, #plots do
    local plot = plots[i]
    
    -- local id, _, _, _, rank = C_Garrison.GetOwnedBuildingInfoAbbrev(plot.id)
    local buildingID, buildingName, texturePrefix, icon, description, rank, currencyID, currencyAmount, goldAmount, timeRequirement, needsPlan, isPreBuilt, possSpecs, upgrades, canUpgrade, isMaxLevel, hasFollowerSlot, knownSpecs, currentSpec, specCooldown, isBeingBuilt, timeStarted, buildDuration, timeRemainingText, canCompleteBuild = C_Garrison.GetOwnedBuildingInfo(plot.id)
    if buildingID then
      local info = {}
      
      info.plot = plot.id
      info.rank = rank
      info.currencyID = currencyID
      info.currencyAmount = currencyAmount
      info.goldAmount = goldAmount
      info.timeRequirement = timeRequirement
      info.needsPlan = needsPlan
      info.canUpgrade = canUpgrade
      info.isMaxLevel = isMaxLevel
      info.hasFollowerSlot = hasFollowerSlot
      info.isBeingBuilt = isBeingBuilt
      info.timeStarted = timeStarted
      info.buildDuration = buildDuration
      info.canCompleteBuild = canCompleteBuild

      buildings[buildingID] = info
    end
  end
  Zaddon.db.global.char[Zaddon.charKey].Buildings = parse:JSONEncode(buildings)
end

--http://wow.gamepedia.com/API_C_Garrison.GetFollowers  
local function ScanFollowers()
  local followersList = C_Garrison.GetFollowers()
  if not followersList then return end

  local followers = {}
  
  -- = C_Garrison.GetFollowerNameByID(id)
  
  local name    -- follower name
  local link    -- follower link
  local id      -- follower id
  local rarity, level, iLevel, ability1, ability2, ability3, ability4, trait1, trait2, trait3, trait4
  
  local numFollowers = 0    -- number of followers
  
  local function IncTableIndex(tableName, index)
    if not tableName[index] then
      tableName[index] = 0
    end
    tableName[index] = tableName[index] + 1   
  end
  
  local isInactive    -- is a follower inactive or not ?
  
  for k, follower in pairs(followersList) do

    name = follower.name
    id = follower.followerID    -- by default, the id should be this one (numeric)
    
    isInactive = nil
    if type(follower.followerID) == "string" then -- if the type is string, it's a GUID
      local status = C_Garrison.GetFollowerStatus(follower.followerID)
      if status and status == GARRISON_FOLLOWER_INACTIVE then
        isInactive = true
      end
    end
    
    if follower.isCollected then
      -- if the follower is collected, the id will be a GUID (string)
      -- therefore, it has to be extracted from the link
      -- also, the link is only valid for collected followers, otherwise it is nil
      link = C_Garrison.GetFollowerLink(follower.followerID)
      id, rarity, level, iLevel, ability1, ability2, ability3, ability4, trait1, trait2, trait3, trait4 = link:match("garrfollower:(%d+):(%d+):(%d+):(%d+):(%d+):(%d+):(%d+):(%d+):(%d+):(%d+):(%d+):(%d+)")
      id = tonumber(id)
      
      local info = {}
      
      info.id = id
      info.xp = follower.xp
      info.levelXP = follower.levelXP
      info.link = link
      info.isInactive = isInactive
      -- followers[name] = info
      
      -- Stats
      numFollowers = numFollowers + 1
      followers[numFollowers] = info
    end
  end
  
  Zaddon.db.global.char[Zaddon.charKey].Followers = parse:JSONEncode(followers)
  Zaddon.db.global.char[Zaddon.charKey].lastUpdate = time()
end

local function ScanResourceCollectionTime()
  Zaddon.db.global.char[Zaddon.charKey].lastResourceCollection = time()
end

local function ScanActiveMissionInfo(missionID)
  if type(missionID) ~= "number" then return end
  
  -- Save the mission start time separately.
  -- The list of active missions does not provide a duration in seconds (only text) or 
  -- a timestamp for the actual start time of that mission.. so keep track of it manually

  -- Note: the times are kept in a separate table
  --  this ensures that if the events GARRISON_MISSION_STARTED & GARRISON_MISSION_LIST_UPDATE 
  --  are triggered in a different order, the system will not fail.

  local missionInfo = Zaddon.db.global.char[Zaddon.charKey].ActiveMissionsInfo  
  if type(missionInfo) ~= "table" then
    missionInfo = parse:JSONDecode(missionInfo)
  end
  local info = {}
  
  info.id = missionID
  info.startTime = time()
  local _, _, _, successChance = C_Garrison.GetPartyMissionInfo(missionID)
  info.successChance = successChance
  
  missionInfo[missionID] = info
  
  Zaddon.db.global.char[Zaddon.charKey].ActiveMissionsInfo = parse:JSONEncode(missionInfo)
end

local function ScanActiveMissions()
  local missionsList = C_Garrison.GetInProgressMissions()
  if not missionsList then return end
  
  local missions = {}
  local numMissions = 0
  --wipe(missions)
  
  for k, mission in pairs(missionsList) do
    local info = {}
    local id = mission.missionID

    local _, _, _, successChance = C_Garrison.GetPartyMissionInfo(id)
    
    info.id = id
    info.successChance = successChance
    info.cost = mission.cost
    info.durationSeconds = mission.durationSeconds
    info.level = mission.level
    info.iLevel = mission.iLevel
    info.type = mission.type
    info.typeAtlas = mission.typeAtlas
    info.inProgress = mission.inProgress
    info.isRare = mission.isRare
    info.missionEndTime = mission.missionEndTime
    info.name = mission.name
    info.numFollowers = mission.numFollowers
    info.numRewards = mission.numRewards
      
    info.rewards = {}
    if mission.rewards then
      for _, reward in pairs(mission.rewards) do
        local rtmp = {}
        if reward.currencyID then
          rtmp.currencyID = reward.currencyID
          rtmp.quantity = reward.quantity
        elseif reward.followerXP then
          rtmp.followerXP = reward.followerXP
        elseif reward.itemID then
          rtmp.itemID = reward.itemID
          rtmp.quantity = reward.quantity
        end
        table.insert(info.rewards, rtmp)
      end
    end
      
      
      --[[
    ]]--
        
    if mission.followers then
      info.followers = {}
      for _, followerGUID in pairs(mission.followers) do
        local link = C_Garrison.GetFollowerLink(followerGUID)
        local id = link:match("garrfollower:(%d+)")
        table.insert(info.followers, tonumber(id))
      end
    end
    
    --ScanActiveMissionInfo(id)

    -- rewards + type : C_Garrison.GetMissionRewardInfo
    numMissions = numMissions + 1
    missions[numMissions] = info
  end

  Zaddon.db.global.char[Zaddon.charKey].lastUpdate = time()
  Zaddon.db.global.char[Zaddon.charKey].ActiveMissions = parse:JSONEncode(missions)
end


-- *** Event Handlers ***
local function OnFollowerAdded()
  ScanFollowers()
end

local function OnFollowerListUpdate()
  ScanFollowers()
end

local function OnFollowerRemoved()
  ScanFollowers()
end

local function OnGarrisonBuildingActivated()
  ScanBuildings()
end

local function OnGarrisonBuildingUpdate()
  ScanBuildings()
end

local function OnGarrisonBuildingRemoved()
  ScanBuildings()
end

local function OnShowLootToast(event, lootType, link, quantity, specID, sex, isPersonal, lootSource)
  if lootType ~= "currency" then return end
  
  -- From AlertFrames.lua
  -- local LOOT_SOURCE_GARRISON_CACHE = 10
  
  -- make sure it is garrison resources
  if link and link:match("currency:824") and lootSource == 10 then  
    ScanResourceCollectionTime()
  end
end

local function OnGarrisonMissionListUpdate()
  ScanActiveMissions()
end

local function OnGarrisonMissionNPCOpened()
  ScanActiveMissions()
end

local function OnGarrisonMissionNPCClosed()
  ScanActiveMissions()
end

local function OnGarrisonMissionStarted(event, missionID)
  ScanActiveMissionInfo(missionID)
end

local function OnGarrisonMissionFinished()
  ScanActiveMissions()
end

function Zaddon:OnInitialize()
  Zaddon.db = LibStub("AceDB-3.0"):New(addonName .. "DB", AddonDB_Defaults)
  
  Zaddon.realmSlug = string.lower(GetRealmName())
  Zaddon.char = UnitName("player")
  Zaddon.charKey = format("%s.%s", Zaddon.realmSlug, Zaddon.char)

  Zaddon.db.global.char[Zaddon.charKey].ActiveMissions = {}
  Zaddon.db.global.char[Zaddon.charKey].ActiveMissionsInfo = {}
  --Zaddon.db:ResetDB("Default")
  --[[Zaddon.db.global.char[Zaddon.charKey] = { 
        lastUpdate = nil,
        lastResourceCollection = nil,
        Buildings = {},
        Followers = {},
        ActiveMissions = {},
        ActiveMissionsInfo = {},
      }]]
  
end


function Zaddon:OnEnable()
  -- Followers
  Zaddon:RegisterEvent("GARRISON_FOLLOWER_ADDED", OnFollowerAdded)
  Zaddon:RegisterEvent("GARRISON_FOLLOWER_LIST_UPDATE", OnFollowerListUpdate)
  Zaddon:RegisterEvent("GARRISON_FOLLOWER_REMOVED", OnFollowerRemoved)
  
  -- Buildings
  Zaddon:RegisterEvent("GARRISON_BUILDING_ACTIVATED", OnGarrisonBuildingActivated)
  Zaddon:RegisterEvent("GARRISON_BUILDING_UPDATE", OnGarrisonBuildingUpdate)
  Zaddon:RegisterEvent("GARRISON_BUILDING_REMOVED", OnGarrisonBuildingRemoved)
  Zaddon:RegisterEvent("SHOW_LOOT_TOAST", OnShowLootToast)
  
  -- Missions
  Zaddon:RegisterEvent("GARRISON_MISSION_LIST_UPDATE", OnGarrisonMissionListUpdate)
  
  Zaddon:RegisterEvent("GARRISON_MISSION_NPC_OPENED", OnGarrisonMissionNPCOpened)
  Zaddon:RegisterEvent("GARRISON_MISSION_NPC_CLOSED", OnGarrisonMissionNPCClosed)
  Zaddon:RegisterEvent("GARRISON_MISSION_STARTED", OnGarrisonMissionStarted)
  Zaddon:RegisterEvent("GARRISON_MISSION_FINISHED", OnGarrisonMissionFinished)
  
  ScanBuildings()
  ScanFollowers()
  ScanActiveMissions()
  
  --ClearActiveMissionsInfo()
end